@@ -36,11 +36,17 @@ __all__ = ["C", "guess_categorical", "CategoricalSniffer",
 import numpy as np
 import six
 from patsy import PatsyError
-from patsy.state import stateful_transform
 from patsy.util import (SortAnythingKey,
-                        have_pandas, have_pandas_categorical,
                         safe_scalar_isnan,
-                        iterable)
+                        iterable,
+                        have_pandas, have_pandas_categorical,
+                        have_pandas_categorical_dtype,
+                        safe_is_pandas_categorical,
+                        pandas_Categorical_from_codes,
+                        pandas_Categorical_categories,
+                        pandas_Categorical_codes,
+                        safe_issubdtype,
+                        no_pickling, assert_no_pickling)
 
 if have_pandas:
     import pandas
@@ -53,6 +59,8 @@ class _CategoricalBox(object):
         self.contrast = contrast
         self.levels = levels
 
+    __getstate__ = no_pickling
+
 def C(data, contrast=None, levels=None):
     """
     Marks some `data` as being categorical, and specifies how to interpret
@@ -112,19 +120,24 @@ def test_C():
     assert c4.contrast == "NEW CONTRAST"
     assert c4.levels == "LEVELS"
 
+    assert_no_pickling(c4)
+
 def guess_categorical(data):
-    if have_pandas_categorical and isinstance(data, pandas.Categorical):
+    if safe_is_pandas_categorical(data):
         return True
     if isinstance(data, _CategoricalBox):
         return True
     data = np.asarray(data)
-    if np.issubdtype(data.dtype, np.number):
+    if safe_issubdtype(data.dtype, np.number):
         return False
     return True
 
 def test_guess_categorical():
     if have_pandas_categorical:
-        assert guess_categorical(pandas.Categorical.from_array([1, 2, 3]))
+        c = pandas.Categorical([1, 2, 3])
+        assert guess_categorical(c)
+        if have_pandas_categorical_dtype:
+            assert guess_categorical(pandas.Series(c))
     assert guess_categorical(C([1, 2, 3]))
     assert guess_categorical([True, False])
     assert guess_categorical(["a", "b"])
@@ -135,6 +148,20 @@ def test_guess_categorical():
     assert not guess_categorical([1.0, 2.0, 3.0])
     assert not guess_categorical([1.0, 2.0, 3.0, np.nan])
 
+def _categorical_shape_fix(data):
+    # helper function
+    # data should not be a _CategoricalBox or pandas Categorical or anything
+    # -- it should be an actual iterable of data, but which might have the
+    # wrong shape.
+    if hasattr(data, "ndim") and data.ndim > 1:
+        raise PatsyError("categorical data cannot be >1-dimensional")
+    # coerce scalars into 1d, which is consistent with what we do for numeric
+    # factors. (See statsmodels/statsmodels#1881)
+    if (not iterable(data)
+        or isinstance(data, (six.text_type, six.binary_type))):
+        data = [data]
+    return data
+
 class CategoricalSniffer(object):
     def __init__(self, NA_action, origin=None):
         self._NA_action = NA_action
@@ -154,11 +181,6 @@ class CategoricalSniffer(object):
         if hasattr(data, "contrast"):
             self._contrast = data.contrast
         # returns a bool: are we confident that we found all the levels?
-        if have_pandas_categorical and isinstance(data, pandas.Categorical):
-            # pandas.Categorical has its own NA detection, so don't try to
-            # second-guess it.
-            self._levels = tuple(data.levels)
-            return True
         if isinstance(data, _CategoricalBox):
             if data.levels is not None:
                 self._levels = tuple(data.levels)
@@ -166,11 +188,19 @@ class CategoricalSniffer(object):
             else:
                 # unbox and fall through
                 data = data.data
+        if safe_is_pandas_categorical(data):
+            # pandas.Categorical has its own NA detection, so don't try to
+            # second-guess it.
+            self._levels = tuple(pandas_Categorical_categories(data))
+            return True
         # fastpath to avoid doing an item-by-item iteration over boolean
         # arrays, as requested by #44
-        if hasattr(data, "dtype") and np.issubdtype(data.dtype, np.bool_):
+        if hasattr(data, "dtype") and safe_issubdtype(data.dtype, np.bool_):
             self._level_set = set([True, False])
             return True
+
+        data = _categorical_shape_fix(data)
+
         for value in data:
             if self._NA_action.is_categorical_NA(value):
                 continue
@@ -187,6 +217,8 @@ class CategoricalSniffer(object):
         # would be too. Otherwise we need to keep looking.
         return self._level_set == set([True, False])
 
+    __getstate__ = no_pickling
+
 def test_CategoricalSniffer():
     from patsy.missing import NAAction
     def t(NA_types, datas, exp_finish_fast, exp_levels, exp_contrast=None):
@@ -199,20 +231,28 @@ def test_CategoricalSniffer():
             else:
                 assert not exp_finish_fast
         assert sniffer.levels_contrast() == (exp_levels, exp_contrast)
-    
+
     if have_pandas_categorical:
-        t([], [pandas.Categorical.from_array([1, 2, None])],
-          True, (1, 2))
-        # check order preservation
-        t([], [pandas.Categorical([1, 0], ["a", "b"])],
-          True, ("a", "b"))
-        t([], [pandas.Categorical([1, 0], ["b", "a"])],
-          True, ("b", "a"))
-        # check that if someone sticks a .contrast field onto a Categorical
-        # object, we pick it up:
-        c = pandas.Categorical.from_array(["a", "b"])
-        c.contrast = "CONTRAST"
-        t([], [c], True, ("a", "b"), "CONTRAST")
+        # We make sure to test with both boxed and unboxed pandas objects,
+        # because we used to have a bug where boxed pandas objects would be
+        # treated as categorical, but their levels would be lost...
+        preps = [lambda x: x,
+                 C]
+        if have_pandas_categorical_dtype:
+            preps += [pandas.Series,
+                      lambda x: C(pandas.Series(x))]
+        for prep in preps:
+            t([], [prep(pandas.Categorical([1, 2, None]))],
+              True, (1, 2))
+            # check order preservation
+            t([], [prep(pandas_Categorical_from_codes([1, 0], ["a", "b"]))],
+              True, ("a", "b"))
+            t([], [prep(pandas_Categorical_from_codes([1, 0], ["b", "a"]))],
+              True, ("b", "a"))
+            # check that if someone sticks a .contrast field onto our object
+            obj = prep(pandas.Categorical(["a", "b"]))
+            obj.contrast = "CONTRAST"
+            t([], [obj], True, ("a", "b"), "CONTRAST")
 
     t([], [C([1, 2]), C([3, 2])], False, (1, 2, 3))
     # check order preservation
@@ -232,11 +272,14 @@ def test_CategoricalSniffer():
     levels, _ = sniffer.levels_contrast()
     assert set(levels) == set([None, 1])
 
-    # bool special case
+    # bool special cases
     t(["None", "NaN"], [C([True, np.nan, None])],
       True, (False, True))
     t([], [C([10, 20]), C([False]), C([30, 40])],
       False, (False, True, 10, 20, 30, 40))
+    # exercise the fast-path
+    t([], [np.asarray([True, False]), ["foo"]],
+      True, (False, True))
 
     # check tuples too
     t(["None", "NaN"], [C([("b", 2), None, ("a", 1), np.nan, ("c", None)])],
@@ -245,42 +288,53 @@ def test_CategoricalSniffer():
     # contrasts
     t([], [C([10, 20], contrast="FOO")], False, (10, 20), "FOO")
 
+    # no box
+    t([], [[10, 30], [20]], False, (10, 20, 30))
+    t([], [["b", "a"], ["a"]], False, ("a", "b"))
+
+    # 0d
+    t([], ["b"], False, ("b",))
+
+    import pytest
+
     # unhashable level error:
-    from nose.tools import assert_raises
     sniffer = CategoricalSniffer(NAAction())
-    assert_raises(PatsyError, sniffer.sniff, [{}])
+    pytest.raises(PatsyError, sniffer.sniff, [{}])
+
+    # >1d is illegal
+    pytest.raises(PatsyError, sniffer.sniff, np.asarray([["b"]]))
 
 # returns either a 1d ndarray or a pandas.Series
 def categorical_to_int(data, levels, NA_action, origin=None):
     assert isinstance(levels, tuple)
     # In this function, missing values are always mapped to -1
-    if have_pandas_categorical and isinstance(data, pandas.Categorical):
-        data_levels_tuple = tuple(data.levels)
+
+    if safe_is_pandas_categorical(data):
+        data_levels_tuple = tuple(pandas_Categorical_categories(data))
         if not data_levels_tuple == levels:
             raise PatsyError("mismatching levels: expected %r, got %r"
                              % (levels, data_levels_tuple), origin)
         # pandas.Categorical also uses -1 to indicate NA, and we don't try to
         # second-guess its NA detection, so we can just pass it back.
-        return data.labels
+        return pandas_Categorical_codes(data)
+
     if isinstance(data, _CategoricalBox):
         if data.levels is not None and tuple(data.levels) != levels:
             raise PatsyError("mismatching levels: expected %r, got %r"
                              % (levels, tuple(data.levels)), origin)
         data = data.data
-    if hasattr(data, "shape") and len(data.shape) > 1:
-        raise PatsyError("categorical data must be 1-dimensional",
-                         origin)
-    if (not iterable(data)
-        or isinstance(data, (six.text_type, six.binary_type))):
-        raise PatsyError("categorical data must be an iterable container")
+
+    data = _categorical_shape_fix(data)
+
     try:
         level_to_int = dict(zip(levels, range(len(levels))))
     except TypeError:
         raise PatsyError("Error interpreting categorical data: "
                          "all items must be hashable", origin)
+
     # fastpath to avoid doing an item-by-item iteration over boolean arrays,
     # as requested by #44
-    if hasattr(data, "dtype") and np.issubdtype(data.dtype, np.bool_):
+    if hasattr(data, "dtype") and safe_issubdtype(data.dtype, np.bool_):
         if level_to_int[False] == 0 and level_to_int[True] == 1:
             return data.astype(np.int_)
     out = np.empty(len(data), dtype=int)
@@ -315,7 +369,7 @@ def categorical_to_int(data, levels, NA_action, origin=None):
     return out
 
 def test_categorical_to_int():
-    from nose.tools import assert_raises
+    import pytest
     from patsy.missing import NAAction
     if have_pandas:
         s = pandas.Series(["a", "b", "c"], index=[10, 20, 30])
@@ -323,28 +377,36 @@ def test_categorical_to_int():
         assert np.all(c_pandas == [0, 1, 2])
         assert np.all(c_pandas.index == [10, 20, 30])
         # Input must be 1-dimensional
-        assert_raises(PatsyError,
+        pytest.raises(PatsyError,
                       categorical_to_int,
                       pandas.DataFrame({10: s}), ("a", "b", "c"), NAAction())
     if have_pandas_categorical:
-        cat = pandas.Categorical([1, 0, -1], ("a", "b"))
-        conv = categorical_to_int(cat, ("a", "b"), NAAction())
-        assert np.all(conv == [1, 0, -1])
-        # Trust pandas NA marking
-        cat2 = pandas.Categorical([1, 0, -1], ("a", "None"))
-        conv2 = categorical_to_int(cat, ("a", "b"), NAAction(NA_types=["None"]))
-        assert np.all(conv2 == [1, 0, -1])
-        # But levels must match
-        assert_raises(PatsyError,
-                      categorical_to_int,
-                      pandas.Categorical([1, 0], ("a", "b")),
-                      ("a", "c"),
-                      NAAction())
-        assert_raises(PatsyError,
-                      categorical_to_int,
-                      pandas.Categorical([1, 0], ("a", "b")),
-                      ("b", "a"),
-                      NAAction())
+        constructors = [pandas_Categorical_from_codes]
+        if have_pandas_categorical_dtype:
+            def Series_from_codes(codes, categories):
+                c = pandas_Categorical_from_codes(codes, categories)
+                return pandas.Series(c)
+            constructors.append(Series_from_codes)
+        for con in constructors:
+            cat = con([1, 0, -1], ("a", "b"))
+            conv = categorical_to_int(cat, ("a", "b"), NAAction())
+            assert np.all(conv == [1, 0, -1])
+            # Trust pandas NA marking
+            cat2 = con([1, 0, -1], ("a", "None"))
+            conv2 = categorical_to_int(cat, ("a", "b"),
+                                       NAAction(NA_types=["None"]))
+            assert np.all(conv2 == [1, 0, -1])
+            # But levels must match
+            pytest.raises(PatsyError,
+                          categorical_to_int,
+                          con([1, 0], ("a", "b")),
+                          ("a", "c"),
+                          NAAction())
+            pytest.raises(PatsyError,
+                          categorical_to_int,
+                          con([1, 0], ("a", "b")),
+                          ("b", "a"),
+                          NAAction())
 
     def t(data, levels, expected, NA_action=NAAction()):
         got = categorical_to_int(data, levels, NA_action)
@@ -360,30 +422,31 @@ def test_categorical_to_int():
     t(["a", "b", "a"], ("a", "d", "z", "b"), [0, 3, 0])
     t([("a", 1), ("b", 0), ("a", 1)], (("a", 1), ("b", 0)), [0, 1, 0])
 
-    assert_raises(PatsyError, categorical_to_int,
+    pytest.raises(PatsyError, categorical_to_int,
                   ["a", "b", "a"], ("a", "c"), NAAction())
 
     t(C(["a", "b", "a"]), ("a", "b"), [0, 1, 0])
     t(C(["a", "b", "a"]), ("b", "a"), [1, 0, 1])
     t(C(["a", "b", "a"], levels=["b", "a"]), ("b", "a"), [1, 0, 1])
     # Mismatch between C() levels and expected levels
-    assert_raises(PatsyError, categorical_to_int,
+    pytest.raises(PatsyError, categorical_to_int,
                   C(["a", "b", "a"], levels=["a", "b"]),
                   ("b", "a"), NAAction())
 
+    # ndim == 0 is okay
+    t("a", ("a", "b"), [0])
+    t("b", ("a", "b"), [1])
+    t(True, (False, True), [1])
+
     # ndim == 2 is disallowed
-    assert_raises(PatsyError, categorical_to_int,
+    pytest.raises(PatsyError, categorical_to_int,
                   np.asarray([["a", "b"], ["b", "a"]]),
                   ("a", "b"), NAAction())
-    # ndim == 0 is disallowed likewise
-    assert_raises(PatsyError, categorical_to_int,
-                  "a",
-                  ("a", "b"), NAAction())
 
     # levels must be hashable
-    assert_raises(PatsyError, categorical_to_int,
+    pytest.raises(PatsyError, categorical_to_int,
                   ["a", "b"], ("a", "b", {}), NAAction())
-    assert_raises(PatsyError, categorical_to_int,
+    pytest.raises(PatsyError, categorical_to_int,
                   ["a", "b", {}], ("a", "b"), NAAction())
 
     t(["b", None, np.nan, "a"], ("a", "b"), [1, -1, -1, 0],
@@ -395,7 +458,7 @@ def test_categorical_to_int():
 
     # Smoke test for the branch that formats the ellipsized list of levels in
     # the error message:
-    assert_raises(PatsyError, categorical_to_int,
+    pytest.raises(PatsyError, categorical_to_int,
                   ["a", "b", "q"],
                   ("a", "b", "c", "d", "e", "f", "g", "h"),
                   NAAction())
